#ifndef AATREE_H
#define AATREE_H

//==========================================================
//NOTE: This one needs some more work and some testing.
// A AA-Tree is like a AVL, or Red/Black tree, but it 
// imposes a couple of restrictions that make the coding
// a little simpler and the run-time slightly better.
//
// FindNode() needs to be completed.
//==========================================================

#include "binarynode.h"

#define Level fBalanceInfo;

template <class T>
class AATree
{
public:
				aaTree();
				~aaTree() {FreeTree(fRoot); delete NilNode;};

	bool	Add(const T value) {return Add(value, fRoot);};
	bool	Remove(const T value) {return Remove(value, fRoot);};
	
	binaryNode<T> *	FindMinNode(binaryNode<T> *) const;
	binaryNode<T> * FindMaxNode(binaryNode<T> *) const;
	binaryNode<T> * FindNode(const T value);
	
	const T &	FindMin() const;
	const T &	FindMax() const;
	
	const T &	Find(const T value);
	bool		WasFound() const {return fCurrent != NilNode;};
	
	void		MakeEmpty() {FreeTree(fRoot); fRoot = NilNode;};
	
protected:
	binaryNode<T> *fRoot;
	binaryNode<T> *NilNode;
	
	// Result of last find, definitely not thread safe
	binaryNode<T> *fCurrent;
	
	// Utility routines
	void	Skew(binaryNode<T> * &);
	void	Split(binaryNode<T> * &);
	
	void	FreeTree(binaryNode<T> *);
	
	bool	Add(const T, binaryNode<T> * &);
	bool	Remove(const T, binaryNode<T> * &);

private:
	const aaTree & operator=(const aaTree<T> &);
	
};

//--------------------------------------------------------------
//--------------------------------------------------------------

template <class T>
aaTree<T>::aaTree()
{
	NilNode = new binaryNode<T>;
	fRoot = NilNode;
}

template <class T>
bool
aaTree<T>::Add(const T value, binaryNode<T> * &node)
{
	if (NilNode == node)
	{
		node = new binaryNode<T>(value, NilNode, NilNode);
		return true;
	} else if (value < node->fValue)
		Add(value, node->fLeft);
	else if (node->fValue < value)
		Add(value, node->fRight);
	else
		return 0;
		
	Skew(node);
	Split(node);
	
	return true;
		
}

template <class T>
bool
aaTree<T>::Remove(const T value, binaryNode<T> * &node)
{
	static bool ItemFound = false;
	static binaryNode<T> *DeletePtr;
	static binaryNode<T> *LastPtr;
	
	if (node != NilNode)
	{
		LastPtr = node;
		if (value < node->fValue)
			Remove (value, node->fLeft)
		else
		{
			DeletePtr = node;
			Remove(value, node->fRight);
		}
		
		// Remove if at bottom of tree and value is there
		if (node == LastPtr)
		{
			if (DeletePtr != NilNode && value == DeletePtr->fValue)
			{
				DeletePtr->fValue = node->fValue;
				DeletePtr = NilNode;
				node = node->fRight;
				delete LastPtr;
				ItemFound = true;
			} else
				ItemFound = false;
		} else if ((node->fLeft->fBalanceInfo < node->fBalanceInfo-1) ||
					(node->fRight->fBalanceInfo < node->fBalanceInfo-1))
		{
			if (node->fRight->fBalanceInfo > --node->fBalanceInfo)
				node->fRight->fBalanceInfo = node->fBalanceInfo;
				
			Skew(node);
			Skew(node->fRight);
			Skew(node->fRight->fRight);
			Split(node);
			Split(node->fRight);
		}
	}
	
	return ItemFound;
}

template <class T>
const binaryNode<T> *
aaTree<T>::FindMinNode(binaryNode<T> *node) const
{
	if (node != 0)
		while(node->fLeft != 0)
			node = node->fLeft;
			
	return node;
}

template <class T>
const binaryNode<T> *
aaTree<T>::FindMaxNode(binaryNode<T> *node) const
{
	if (node != 0)
		while(node->fRight != 0)
			node = node->fRight;
			
	return node;
}

template <class T>
const T &	
aaTree<T>::FindMin() const
{
	binaryNode<T> *aNode = FindMinNode(fRoot);
	return aNode->fValue;
}

template <class T>
const T &	
aaTree<T>::FindMax() const
{
	binaryNode<T> *aNode = FindMaxNode(fRoot);
	return aNode->fValue;
}

template <class T>
binaryNode<T> *	
aaTree<T>::FindNode(const T value)
{
}

template <class T>
void	
aaTree<T>::FreeTree(binaryNode<T> *tree)
{
	if (tree != 0 && (tree != NilNode))
	{
		FreeTree(tree->fLeft);
		FreeTree(tree->fRight);
		delete tree;
	}
}

template <class T>
void
aaTree<T>::Skew(binaryNode<T> * &node)
{
	if (node->fLeft->fBalanceInfo == node->fBalanceInfo)
		node = RotateWithLeftChild(node);
}

template <class T>
void
aaTree<T>::Split(binaryNode<T> * &node)
{
	if (node->fRight->fRight->fBalanceInfo == node->fBalanceInfo)
	{
		node = RotateWithRightChild(node);
		node->fBalanceInfo++;
	}
}

#endif
