#ifndef binaryNode_H
#define binaryNode_H

//==========================
// NEED TO 	IMPLEMENT
// AVL-Tree
// AA-Tree
// RedBlack-Tree
// BTree
//
// Trees that use this
// splayTree
// searchTree
//==========================

//====================================================================
// Template Interface: binaryNode<T>
//
// The binary node is a very simple object that is at the root
// of all tree implementations.  It simply maintains and gives access
// to a value, a left child and a right child.
// struct: binaryNode
//
// Note: We want everything to be public for quick fast and easy
// access.  Since accessors simply set and get values anyway, there
// is really no harm in exposing the instance variables.
//====================================================================

template <class T> 
struct binaryNode
{
				binaryNode();
				binaryNode(const binaryNode<T> &);
				binaryNode(T aValue);
				binaryNode(T aValue, binaryNode<T>* left, binaryNode<T>* right);
	virtual		~binaryNode();

				// Allow assignment
				binaryNode<T> & operator = (const binaryNode<T> &rhs);
				
				binaryNode<T>*	Copy() const;
				void			Release();

				binaryNode<T>*	Left() const {return fLeft;};
				binaryNode<T>*	Right() const {return fRight;};

				void				SetLeft(binaryNode<T> *aValue) {fLeft = aValue;};
				void				SetRight(binaryNode<T> *aValue) {fRight = aValue;};


				unsigned long	Size(const binaryNode<T> *) const ;	// number of elements in whole tree
				long			Height(const binaryNode<T> *) const;
				
				
				T				fValue;		// Value stored in this node
				binaryNode<T>*	fLeft;		// pointer to left sub-tree
				binaryNode<T>*	fRight;		// pointer to right sub-tree
				long			fSize;
				long			fBalanceInfo;
};


//====================================================================
// Template Implementation: binaryNode<T>
// Implementation
//
//====================================================================
template <class T>
binaryNode<T>::binaryNode()
{
	fLeft = 0;
	fRight = 0;
	fSize = 1;
	fBalanceInfo = 1;
}

template <class T>
binaryNode<T>::binaryNode(const binaryNode<T> &rhs)
	: fValue(rhs.fValue)
{
	fLeft = rhs.fLeft->Copy();
	fRight = rhs.fRight->Copy();
	fSize = rhs.fSize;
	fBalanceInfo = rhs.fBalanceInfo;
}

template <class T>
binaryNode<T>::binaryNode(T aValue)
	: fValue(aValue)
{
	fLeft = 0;
	fRight = 0;
	fSize = 1;
	fBalanceInfo = 1;
}

template <class T>
binaryNode<T>::binaryNode(T aValue, binaryNode<T>* left, binaryNode<T>* right)
	:fValue(aValue),
	fLeft(0),
	fRight(0)
{
	fLeft = left;
	fRight = right;
	fSize = 1;
	fBalanceInfo = 1;
}

template <class T>
binaryNode<T>::~binaryNode()
{
	// We don't want to call release here because it should have
	// already been called before you get here.  So there's nothing
	// else to do in here.
	//Release();
}

template <class T>
binaryNode<T> &
binaryNode<T>::operator = (const binaryNode<T> &rhs)
{
	// If we are assigning to ourself, then just return
	if (&rhs == this)
		return *this;
		
	// Get rid of old children
	if (fLeft)
		fLeft->Release();
	if (fRight)
		fRight->Release();

	// Assign new value, and make copies if children
	fValue = rhs.fValue;
	fLeft = rhs.fLeft->Copy();
	fRight = rhs.fRight->Copy();
	
	return *this;
}

template <class T>
binaryNode<T>*
binaryNode<T>::Copy() const
{
	binaryNode<T> * newLeft=0;
	binaryNode<T> * newRight=0;

	if (fLeft != 0)
		newLeft = fLeft->Copy();

	if (fRight != 0)
		newRight = fRight->Copy();

	binaryNode<T> * newNode = new binaryNode<T>(fValue, newLeft, newRight);
	assert(newNode != 0);

	return newNode;
}

template <class T>
void
binaryNode<T>::Release()
{
	if (fLeft)
	{
		if (fLeft != this)
		{
			fLeft->Release();
			delete fLeft;
		}
		fLeft = 0;
	}

	if (fRight)
	{
		if (fRight != this)
		{
			fRight->Release();
			delete fRight;
		}
		fRight = 0;
	}
}

template <class T>
unsigned long
binaryNode<T>::Size(const binaryNode<T> * aTree) const
{
	if (aTree == 0)
		return 0;
	else
		return 1 + Size(aTree->Left()) + Size(aTree->Right());	
}

template <class T>
long
binaryNode<T>::Height(const binaryNode<T> *aTree) const
{
	if (0 == aTree)
		return -1;
	else
	{
		// Why not use a MAX macro or template?  This keeps things
		// simple and doesn't make us include another header.
		long lheight = Height(aTree->Left());
		long rheight = Height(aTree->Right());
		
		return 1 + ((lheight>rheight) ? lheight : rheight);
	}
}

//===================================================================
// Utility functions for binary nodes
//
// These rotation routines are useful for things like AVL-Trees
// and Splay Trees.
//===================================================================

template <class T>
binaryNode<T> *
RotateWithLeftChild(binaryNode<T> *node)
{
	 binaryNode<T> *tempNode = node->Left();
	 node->SetLeft(tempNode->Right());
	 tempNode->SetRight(node);
	 
	 return tempNode;
}

template <class T>
binaryNode<T> *
RotateWithRightChild(binaryNode<T> *node)
{
	 binaryNode<T> *tempNode = node->Right();
	 node->SetRight(tempNode->Left());
	 tempNode->SetLeft(node);
	 
	 return tempNode;
}

template <class T>
binaryNode<T> *
DoubleRotateWithLeftChild(binaryNode<T> *K3)
{
	K3->SetLeft(RotateWithRightChild(K3->Left());
	return RotateWithLeftChild(K3);
}

template <class T>
binaryNode<T> *
DoubleRotateWithRightChild(binaryNode<T> *K1)
{
	K1->SetRight(RotateWithLeftChild(K1->Right());
	return RotateWithRightChild(K1);			
}

#endif
