#ifndef HASHVECTOR_H
#define HASHVECTOR_H

#include "vector.h"

// Forward declarations
template <class H, class T> class hashVector;


//===============================================================
// Template Interface: hashVector<H,T>
//
// H - The type of the key to be hashed
// T - The type of the actual data
//===============================================================

template <class H, class T> 
class hashVector : public vector<T>
{
public:
				hashVector(const unsigned long max, unsigned long (*hashfunc)(H, const unsigned long tableSize));
				hashVector(const unsigned long max, unsigned long (*hashfunc)(H, const unsigned long tableSize), T & defaultValue);
				hashVector(const hashVector<H,T> &);
				
				// The only thing a hashVector does different is
				// the subscript operator.
				T & operator [] (const H idx);
				
protected:
	unsigned long	(*fHashFunction)(H, const unsigned long tableSize);
	unsigned long	fTableSize;
	
private:
				hashVector();
				
};



//===============================================================
//
//===============================================================

template <class H, class T>
hashVector<H,T>::hashVector(const unsigned long max, unsigned long (*f)(H, const unsigned long tableSize))
	: vector<T>(max),
	fHashFunction(f),
	fTableSize(max)
{
}

template <class H, class T>
hashVector<H,T>::hashVector(const unsigned long max, unsigned long (*f)(H, const unsigned long tableSize), T & defaultValue)
	: vector<T>(max,defaultValue),
	fHashFunction(f),
	fTableSize(max)
{
}

template <class H, class T>
hashVector<H,T>::hashVector(const hashVector<H,T> &rhs)
	: vector<T>(rhs),
	fHashFunction(rhs.fHashFunction)
{
}

template <class H, class T>
T & hashVector<H,T>::operator [] (const H idx)
{
	return vector<T>::operator[](fHashFunction(idx,fTableSize));
}


#endif
