#ifndef LIST_H
#define LIST_H

#include<assert.h>

// Forward declarations
template <class AType> class list;
template <class AType> class link;
template <class AType> class listIterator;


//==============================================
//
//==============================================

template <class AType> class link
{
public:
		link<AType>	*	Insert(AType aValue);



protected:
private:
				link(AType aValue, link<AType> * nextLink);

		link<AType> *	Copy();

	// Data
	AType	fValue;
	link<AType> *	fNextLink;

	// friends
	friend class list<AType>;
	friend class listIterator<AType>;
};

//==============================================
//
//==============================================
template <class AType> class list
{
public:
				list();
				list(const list<AType>& other);
	virtual	~list();

	virtual	void	Add(AType aValue);
	virtual	void	DeleteAll();
				AType	FirstElement() const;
	virtual	bool	Includes(const AType& aValue) const;
			bool	IsEmpty() const;
	virtual	void	RemoveFirst();


protected:
	link<AType>*	fFirstLink;
	
	// friends
	friend class listIterator<AType>;
	
private:
};


//===================================================
// Implementation: link
//===================================================
template <class AType>
link<AType>::link(AType aValue, link<AType>* nextLink)
	: fValue(aValue),
	fNextLink(nextLink)
{
}

template <class AType>
link<AType> *
link<AType>::Insert(AType aValue)
{
	fNextLink = new link<AType>(aValue, fNextLink);
	assert(fNextLink != 0);

	return fNextLink;
}

template <class AType>
link<AType> *
link<AType>::Copy()
{
	link<AType> * newLink;

	if (fNextLink != 0)
		newLink = new link<AType>(fValue, fNextLink->Copy());
	else
		newLink = new link<AType>(fValue, 0);

	assert(newLink != 0);

	return newLink;
}

//===================================================
// Implementation: list
//===================================================
template <class AType>
list<AType>::list()
	: fFirstLink(0)
{

}

template <class AType>
list<AType>::list(const list<AType>& other)
{
	if (other.IsEmpty())
	{
		fFirstLink = 0;
	} else
	{
		fFirstLink = other.fFirstLink->Copy();
	}
}

template <class AType>
void
list<AType>::Add(AType aValue)
{
	fFirstLink = new link<AType>(aValue,fFirstLink);
	assert(fFirstLink!=0);
}

template <class AType>
bool
list<AType>::IsEmpty() const
{
	return (0 == fFirstLink);
}

template <class AType>
AType
list<AType>::FirstElement() const
{
	assert(fFirstLink != 0);
	return fFirstLink->fValue;
}

template <class AType>
void
list<AType>::RemoveFirst()
{
	assert(fFirstLink != 0);

	link<AType> * tmpPtr = fFirstLink;

	fFirstLink = tmpPtr->fNextLink;

	delete tmpPtr;
}

template <class AType>
bool
list<AType>::Includes(const AType& aValue) const
{
	for (link<AType> * aLink = fFirstLink; aLink; aLink = aLink->fNextLink)
	{
		if (aValue == aLink->fValue)
			return true;
	}

	// The value wasn't found, so return 0 (false);
	return false;
}

template <class AType>
list<AType>::~list()
{
	DeleteAll();
}

template <class AType>
void
list<AType>::DeleteAll()
{
	link<AType> * nextLink;
	for (link<AType> * aLink = fFirstLink; aLink != 0; aLink = nextLink)
	{
		nextLink = aLink->fNextLink;
		aLink->fNextLink = 0;
		delete aLink;
	}

	fFirstLink = 0;
}

#endif
