#ifndef QUEUELIST_H
#define QUEUELIST_H

#include "queue.h"
#include "doubleendedlist.h"

//========================================================
// Class: queueList
//
// This is an implementation of the queue protocol using a
// doubleendedlist as the underlying data structure for storage 
// of the values.
//========================================================

template <class AType> class queueList : public queue<AType>
{
public:
		queueList();
		queueList(const queueList& other);
		
	
	// Implementation of queue protocol
	virtual	void	MakeEmpty();
	virtual AType	Dequeue();
	virtual void	Enqueue(const AType val);
	virtual AType	Front();
	virtual bool		IsEmpty() const;
	

protected:
	doubleEndedList<AType>	fData;		// The sequence that holds the data
};


//===================================================
// Implementation: queueList
//===================================================

template <class AType> 
queueList<AType>::queueList()
{
	// Initialize sequence to being empty
	DeleteAll();
}

template <class AType> 
queueList<AType>::queueList(const queueList& other)
	: fData(other.fData)
{
}

template <class AType> 
void 
queueList<AType>::MakeEmpty()
{
	fData.DeleteAll();
}

template <class AType> 
bool
queueList<AType>::IsEmpty() const
{
	return fData.IsEmpty();
}


template <class AType> 
void
queueList<AType>::Enqueue(const AType val)
{
	fData.AddToEnd(val);
}

template <class AType> 
AType
queueList<AType>::Dequeue()
{
	AType result = fData.FirstElement();
	fData.RemoveFirst();
	return result;
}

template <class AType> 
AType
queueList<AType>::Front()
{
	return fData.FirstElement();
}

#endif
