#ifndef STACKLIST_H
#define STACKLIST_H

#include "stack.h"
#include "list.h"

//========================================================
// Class: stackList
//
// This is an implementation of the stack protocol using a
// list as the underlying data structure for storage of
// the values.
//========================================================

template <class T> class stackList : public stack<T>
{
public:
	stackList();
	stackList(const stackList &);
	
	// stack protocol
	virtual void	Push(T val);
	virtual T		Pop();
	virtual T		Top() const;

	virtual	void	DeleteAll();
	virtual bool	IsEmpty() const;
	
protected:
	list<T>		fData;
	
private:
};




//===============================================

template <class T> 
stackList<T>::stackList()
	: fData()
{
}

template <class T> 
stackList<T>::stackList(const stackList<T> &other)
	: fData(other.fData)
{
}

template <class T> 
void	
stackList<T>::DeleteAll()
{
	fData.DeleteAll();
}

template <class T> 
bool	
stackList<T>::IsEmpty() const
{
	return fData.IsEmpty();
}

template <class T> 
T	
stackList<T>::Pop()
{
	T result = fData.FirstElement();
	
	fData.RemoveFirst();
	
	return result;
}

template <class T> 
void	
stackList<T>::Push(T val)
{
	fData.Add(val);
}

template <class T> 
T	
stackList<T>::Top() const
{
	return fData.FirstElement();
}

#endif
