#ifndef STACKVECTOR_H
#define STACKVECTOR_H

#include "stack.h"
#include "vector.h"

//========================================================
// Class: stackVector
//
// This is an implementation of the stack protocol using a
// vector as the underlying data structure for storage of
// the values.
//========================================================
template <class T> class stackVector : public stack<T>
{
public:
	stackVector(unsigned long size);
	stackVector(const stackVector &);
	
	// stack protocol
	virtual void	Push(AType val);
	virtual AType	Pop();
	virtual AType	Top() const;

	virtual	void	DeleteAll();
	virtual bool	IsEmpty() const;
	
protected:
	vector<T>	fData;
	unsigned long	fNextSlot;
	
private:
};




//===============================================

template <class T> stackVector<T>::stackVector(unsigned long size)
	: fData(size)
{
	DeleteAll();
}

template <class T> stackVector<T>::stackVector(const stackVector<T> &other)
	: fData(other.fData),
	fNextSlot(other.fNextSlot)
{
}

template <class T> 
void	
stackVector<T>::DeleteAll()
{
	fNextSlot = 0;
}

template <class T> 
bool	
stackVector<T>::IsEmpty() const
{
	return fNextSlot == 0;
}

template <class T> 
AType	
stackVector<T>::Pop()
{
	assert(!IsEmpty())
	return fData[--fNextSlot];
}

template <class T> 
void	
stackVector<T>::Push(AType val)
{
	if (fNextSlot >= fData.Length())
		fData.SetSize(fData.Length + 5);
	fData[fNextSlot++] = val;
}

template <class T> 
AType	
stackVector<T>::Top() const
{
	assert(!IsEmpty());
	return fData[fNextSlot-1];
}

#endif
