#include "thread.h"

#include <stdio.h>
#include <stdlib.h>

//========================================================
//
//========================================================

static status_t
thread_mainthread(void *data)
{	
	thread *aThread = (thread *)data;
  	return aThread->ThreadMain();
}

thread::thread(const char *aName, long aPriority)
	: fThreadID(-1)
{
	// Create the actual thread
	fThreadID = spawn_thread(thread_mainthread, 
		aName, aPriority, this);
}

thread::~thread()
{
	long exitStatus;
	
	Kill();
	WaitForExit(&exitStatus);
}


status_t
thread::GetInfo(thread_info *info)
{
	return get_thread_info(fThreadID, info);
}

//
// This is the method that sub-classers need to
// override for their main execution.  If you don't
// override this, then your thread won't do much
// more than exit.  It is the only method that is
// absolutely required to override.
status_t 
thread::ThreadMain()
{
	printf("thread:: ThreadMain\n");
	return 0;
}

status_t	
thread::Kill()
{
	return kill_thread(fThreadID);
}

status_t	
thread::Run()
{
	return resume_thread(fThreadID);
}

status_t	
thread::Suspend()
{
	return suspend_thread(fThreadID);
}

status_t	
thread::Rename(const char *newName)
{
	return rename_thread(fThreadID, newName);
}

status_t	
thread::SetPriority(long newPriority)
{
	return set_thread_priority(fThreadID, newPriority);
}

status_t
thread::WaitForExit(long *exitStatus)
{
	return wait_for_thread(fThreadID, exitStatus);
}

/*

status_t	
thread::Snooze(bigtime_t microseconds)
{
	return snooze(microseconds);
}

extern thread_id	FindThread(const char *name); 
extern long			GetNthThreadInfo(team_id tmid, long n, thread_info *info);
*/

