#ifndef VECTORITERATOR_H
#define VECTORITERATOR_H

#include "iterator.h"
#include "vector.h"

//===========================================
// Class Template: vectorIterator
// The concrete template class implementation of PIteratorOver
//===========================================
template<class T> 
class vectorIterator : public iterator<T>
{
public:
				vectorIterator(vector<T>& aSequence);

				// Implement the PIteratorOver protocol
			virtual	bool	init();
			virtual bool	done();
			virtual long	next();
			
			virtual	T	operator()();
			virtual	long	operator++();
			virtual	void	operator=(T newValue);

			// Sequence Iterators have these specifics
			long				operator--();
			unsigned long	Key();

protected:
			unsigned long			fCurrentKey;
			vector<T>&	fData;

private:

};

//===========================================
// Class Template: vectorIterator
// Implementations
//===========================================

template <class T>
vectorIterator<T>::vectorIterator(vector<T>& aSequence)
	: fData(aSequence)
{
	init();
}

// Implement the PIteratorOver protocol
template <class T>
bool
vectorIterator<T>::init()
{
	fCurrentKey = 0;
	return !done();
}


template <class T>
bool
vectorIterator<T>::done()
{
	return !(fCurrentKey < fData.Length());
}


template <class T>
T
vectorIterator<T>::operator()()
{
	return fData[fCurrentKey];
}


template <class T>
long
vectorIterator<T>::next()
{
	fCurrentKey++;
	return !done();
}

template <class T>
long
vectorIterator<T>::operator++()
{
	fCurrentKey++;
	return !done();
}


template <class T>
void
vectorIterator<T>::operator=(T newValue)
{
	fData[fCurrentKey] = newValue;
}


// vector Iterators have these specifics
template <class T>
long
vectorIterator<T>::operator--()
{
	if (fCurrentKey > 0)
		fCurrentKey--;
	return operator!();
}

template <class T>
unsigned long
vectorIterator<T>::Key()
{
	return fCurrentKey();
}

#endif
